import os
import subprocess


class Utilities:
    cwd = os.getcwd()

    @staticmethod
    def findShell():
        try:
            process = subprocess.Popen(
                'bash',
                stderr=subprocess.PIPE,
                stdin=subprocess.DEVNULL,
                stdout=subprocess.DEVNULL,
            )
            _, err = process.communicate()
            if not err:
                return 'bash'
        except:
            pass

        try:
            process = subprocess.Popen(
                'sh',
                stderr=subprocess.PIPE,
                stdin=subprocess.DEVNULL,
                stdout=subprocess.DEVNULL,
            )
            _, err = process.communicate()
            if not err:
                return 'sh'
        except:
            pass

        try:
            process = subprocess.Popen(
                'where git',
                stderr=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stdin=subprocess.DEVNULL,
            )
            out, err = process.communicate()
            if not err:
                out = out.decode('utf-8')
                shellPath = os.path.join(
                    out.split('\n')[0], '..', '..', 'usr', 'bin', 'bash')
                if os.path.exists(shellPath):
                    return shellPath
                shellPath = os.path.join(
                    out.split('\n')[0], '..', '..', 'usr', 'bin', 'sh')
                if os.path.exists(shellPath):
                    return shellPath
        except:
            pass

    @staticmethod
    def excuteCommand(command):
        process = subprocess.Popen(Utilities.findShell(),
                                   stdin=subprocess.PIPE,
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE,
                                   cwd=Utilities.cwd)

        return process.communicate(str.encode(command))

    @staticmethod
    def clearConsole():
        os.system('cls' if os.name == 'nt' else 'clear')



import re
import sys
from os import path
from datetime import datetime
from shutil import rmtree, move
from gitools.editor import Editor
from gitools.utils import Utilities


class Gitools:
    def __init__(self, commit_hash, commit_count, commit_date, commit_message,
                 author_name, author_email):
        if (commit_count is None):
            commit_count = 10

        self.commit_hash = commit_hash
        self.commit_date = commit_date
        self.author_name = author_name
        self.author_email = author_email
        self.commit_count = commit_count
        self.commit_message = commit_message

    def updateDate(self):
        self.checkBackup()
        self.selectCommit()

        if not self.commit_date:
            date = input('Enter New Commit Date: ')
            try:
                self.commit_date = datetime.strptime(
                    date, '%a %b %d %X %Y %z').strftime('%a %b %d %X %Y %z')
            except:
                print(
                    'Enter Valid Date Such As "Fri Jan 1 00:00:00 2021 +0000"')
                self.updateDate()
                return

        command = """git filter-branch -f --env-filter 'if [ $GIT_COMMIT = "COMMIT_HASH" ]
                then
                    export GIT_AUTHOR_DATE="NEW_DATE";
                    export GIT_COMMITTER_DATE="NEW_DATE";
                fi'""".replace("COMMIT_HASH", self.commit_hash).replace(
            "NEW_DATE", self.commit_date)

        _, err = Utilities.excuteCommand(command)

        if not err:
            print("Date Changed Successfully")
        else:
            print("Date Change Failed, Error:\r\n" + err.decode('utf-8'))

    def updateMessage(self):
        self.checkBackup()
        self.selectCommit()

        if not self.commit_message:
            self.commit_message = Editor.input('# Enter New Commit Message')

        command = """git filter-branch --msg-filter 'if [[ $GIT_COMMIT = "COMMIT_HASH" ]]
                    then
                        echo "NEW_MESSAGE"
                    else
                        cat
                    fi'""".replace("COMMIT_HASH", self.commit_hash).replace(
            "NEW_MESSAGE", self.commit_message)

        _, err = Utilities.excuteCommand(command)

        if not err:
            print("Commit Message Changed Successfully")
        else:
            print("Commit Message Change Failed, Error:\r\n" +
                  err.decode('utf-8'))

    def updateAuthor(self):
        self.checkBackup()
        self.selectCommit()

        if not self.author_name:
            self.author_name = input('Enter New Author Name: ').strip()
        if not self.author_email:
            self.author_email = input('Enter New Author Email: ').strip()

        print()

        command = """git filter-branch --env-filter 'if [[ $GIT_COMMIT = "COMMIT_HASH" ]]
                    then
                        export GIT_AUTHOR_NAME="NEW_AUTHOR_NAME"
                        export GIT_AUTHOR_EMAIL="NEW_AUTHOR_EMAIL"
                        export GIT_COMMITTER_NAME="NEW_AUTHOR_NAME"
                        export GIT_COMMITTER_EMAIL="NEW_AUTHOR_EMAIL"
                    fi'""".replace("COMMIT_HASH", self.commit_hash).replace(
            "NEW_AUTHOR_NAME",
            self.author_name).replace("NEW_AUTHOR_EMAIL", self.author_email)

        _, err = Utilities.excuteCommand(command)

        if not err:
            print("Commit Author Changed Successfully")
        else:
            print("Commit Author Change Failed, Error:\r\n" +
                  err.decode('utf-8'))

    def selectCommit(self):
        if not self.commit_hash:
            commits = self.getCommits()
            print('{:15s} {:30s} {:30s} {:30s}'.format('Index', 'Author',
                                                       'Message', 'Date'))
            for i, c in enumerate(commits):
                print('{:15s} {:30s} {:30s} {:30s}'.format(
                    str(i + 1), c[2], c[5], c[4]))

            print()
            index = input('Enter Commit Index: ')
            try:
                self.commit_hash = commits[int(index) - 1][0]
                print()
            except:
                Utilities.clearConsole()
                print(
                    "Enter a valid index or run with like following command\r\n> gitools --hash xxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                )
                print()
                self.selectCommit()

    def getCommits(self):
        out, err = Utilities.excuteCommand('git log -n ' +
                                           str(self.commit_count))

        if err:
            raise Exception("An Error Occurred:\r\n" + err.decode("utf-8"))

        out = out.decode("utf-8")

        return re.findall(
            r'commit\s([a-zA-Z0-9]*)(.*)\nAuthor:\s*([^<]*)<(.*)>\nDate:\s*(.*)\n\n\s*(.*)',
            out)

    def checkBackup(self):
        if path.exists(path.join(Utilities.cwd, '.git', 'refs', 'original')):
            print('Are you sure you want to overwrite backup? [Default=Y/N]')
            res = input()
            if res.lower() == 'n':

                print('Do you want to restore backup? [Y/Default=N]')
                res = input()
                if res.lower() == 'y':
                    self.restoreBackup()
                else:
                    sys.exit(1)

            else:
                self.deleteBackup()

    def deleteBackup(self):
        rmtree(path.join(Utilities.cwd, '.git', 'refs', 'original'))

    def restoreBackup(self):
        rmtree(path.join(Utilities.cwd, '.git', 'refs', 'heads'))
        move(
            path.join(Utilities.cwd, '.git', 'refs', 'original', 'refs',
                      'heads'), path.join(Utilities.cwd, '.git', 'refs'))
        self.deleteBackup()



import argparse
from os import path
from gitools.utils import Utilities
from gitools.gitools import Gitools


def main():
    try:
        parser = argparse.ArgumentParser()
        parser.add_argument("-c",
                            "--count",
                            type=int,
                            required=False,
                            help="number of commit to show")
        parser.add_argument("-ae",
                            "--author-email",
                            type=str,
                            required=False,
                            help="author email")
        parser.add_argument("-an",
                            "--author-name",
                            type=str,
                            required=False,
                            help="author name")
        parser.add_argument("-cd",
                            "--commit-date",
                            type=int,
                            required=False,
                            help="commit date")
        parser.add_argument("-ch",
                            "--commit-hash",
                            type=str,
                            required=False,
                            help="commit hash")
        parser.add_argument("-cm",
                            "--commit-message",
                            type=str,
                            required=False,
                            help="commit message")
        parser.add_argument("-m",
                            "--mode",
                            type=str,
                            required=False,
                            help='command mode')
        parser.add_argument("-i",
                            "--input",
                            type=str,
                            required=False,
                            help="git directory")
        args = parser.parse_args()

        selectMode(args)

        if args.input:
            Utilities.cwd = path.realpath(args.input)

        gitHand = Gitools(commit_count=args.count,
                          commit_hash=args.commit_hash,
                          commit_date=args.commit_date,
                          author_name=args.author_name,
                          author_email=args.author_email,
                          commit_message=args.commit_message)
        if args.mode == 'ud':
            gitHand.updateDate()
        elif args.mode == 'um':
            gitHand.updateMessage()
        elif args.mode == 'ua':
            gitHand.updateAuthor()
        elif args.mode == 'rb':
            gitHand.restoreBackup()
        else:
            parser.print_help()
        pass
    except KeyboardInterrupt:
        print()
        print('Process Canceled')


def selectMode(args):
    modes = [('Update Date', 'ud', 'Update commit date time'),
             ('Update Message', 'um', 'Update commit message'),
             ('Update Author', 'ua', 'Update author information'),
             ('Restore Backup', 'rb', 'Restore Backup')]
    if not args.mode:
        print('{:15s} {:25s} {:10s} {:30s}'.format('Index', 'Mode', 'Flag',
                                                   'Description'))
        for i, mode in enumerate(modes):
            print('{:15s} {:25s} {:10s} {:30s}'.format(*((str(i + 1), ) +
                                                         mode)))
        index = input("Enter Mode Index: ")
        try:
            args.mode = modes[int(index) - 1][1]
            Utilities.clearConsole()
        except:
            Utilities.clearConsole()
            print(
                "Enter a valid index or run with like following command\r\n> gitools -m uh"
            )
            print()
            selectMode(args)


if __name__ == "__main__":
    main()


import os
import tempfile
import subprocess


class Editor:
    @staticmethod
    def input(message):
        tmp = tempfile.mktemp()
        if not message is None:
            with open(tmp, 'a') as f:
                f.write(message)
        process = subprocess.Popen(
            Editor.findEditor() + [tmp],
            stderr=None,
            stdout=None,
        )
        _, _ = process.communicate()

        with open(tmp) as f:
            input = f.readlines()
        os.remove(tmp)

        return ''.join(filter(lambda x: x.strip() != message, input)).strip()

    @staticmethod
    def findEditor():
        try:
            process = subprocess.Popen('nano')
            process.terminate()
            return ['nano', '-t']
        except:
            pass

        try:
            process = subprocess.Popen('vim')
            process.terminate()
            return ['vim', '-n']
        except:
            pass

        try:
            process = subprocess.Popen('vi')
            process.terminate()
            return ['vi', '-n']
        except:
            pass

        try:
            process = subprocess.Popen(
                'where git',
                stderr=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stdin=subprocess.DEVNULL,
            )
            out, err = process.communicate()
            if not err:
                out = out.decode('utf-8')
                out = out.split('\n')[0]
                editorPath = os.path.join(out, '..', '..', 'usr', 'bin',
                                          'nano')
                if os.path.exists(editorPath):
                    return [editorPath, '-t']
                editorPath = os.path.join(out, '..', '..', 'usr', 'bin', 'vim')
                if os.path.exists(editorPath):
                    return [editorPath, '-n']
                editorPath = os.path.join(out, '..', '..', 'usr', 'bin', 'vi')
                if os.path.exists(editorPath):
                    return [editorPath, '-n']
        except:
            pass


import setuptools
from distutils.core import setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(name='Gitools',
      version='1.2.0',
      license="MIT",
      author='AliReza Beigy',
      author_email='AliRezaBeigyKhu@gmail.com',
      entry_points="""
        [console_scripts]
        gitools=gitools.__main__:main
    """,
      python_requires='>=3.6',
      platforms=['nt', 'posix'],
      long_description=long_description,
      packages=setuptools.find_packages(),
      url='https://github.com/AliRezaBeigy/Gitools',
      long_description_content_type="text/markdown",
      description='A handy tool to modify git history',
      keywords='git history filter-branch change-date git-history',
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Intended Audience :: Developers',
          'Topic :: Software Development :: Version Control :: Git',
          'License :: OSI Approved :: MIT License',
      ])
